#include "StdAfx.h"
#include "WASRecvMemo.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"
#include "GTCommonAPI.h"


///////////////////////////////////////////////////////////////////////////////
//// CWASSendNote /////////////////////////////////////////////////////////////

CWASSendNote::CWASSendNote(CWASActor *pActor) : CWASCommand(pActor)
{
	m_pSendNoteData = NULL;
}

CWASSendNote::~CWASSendNote(void)
{
}

void CWASSendNote::ExecuteAsync()
{
	m_eError = wasError_None;
	m_nErrorCode = 0;
	m_strResponseMsg = _T("");

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL || m_pSendNoteData == NULL)
	{
		return;
	}	

	m_eError = wasError_None;

	const char* chTicket		= "ticket";
	const char* chList			= "toIdList";
	const char* chTitle			= "noteTtl";
	const char* chNoteContent	= "noteContent";
	const char* chRsvCheck		= "rsvDsphYn";
	const char* chRsvDate		= "rsvDsphYmdt";
	const char* chTimeZone		= "tmTpCd";
	const char* chAttachID		= "attachId";
	const char* chLang			= "defaultLanguage";
	const char* chParty         = "relRcvrYn";
	const char* chUserID		= "userid";

	USES_UTF8_CONVERSION;

	WSTRING strURL = _T("api-nnote.ncloudbiz.com/api/note/send.nhn");

	m_httpFacade.session().SetURL( W2U8(strURL.c_str()), pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable(chTicket, Conversion::URLdecode((char*)pMessengerActor->GetMessengerTicket()));
	m_httpFacade.session().AddBodyVariable(chList, Conversion::WideToUtf8(m_pSendNoteData->strToList).c_str());
	m_httpFacade.session().AddBodyVariable(chTitle, Conversion::WideToUtf8(m_pSendNoteData->strTitle).c_str());
	m_httpFacade.session().AddBodyVariable(chNoteContent, Conversion::WideToUtf8(m_pSendNoteData->strNoteContent).c_str());
	m_httpFacade.session().AddBodyVariable(chRsvCheck, Conversion::WideToUtf8(m_pSendNoteData->strRsvCheck).c_str());
	m_httpFacade.session().AddBodyVariable(chRsvDate, Conversion::WideToUtf8(m_pSendNoteData->strRsvDate).c_str());
	m_httpFacade.session().AddBodyVariable(chTimeZone, Conversion::WideToUtf8(m_pSendNoteData->strTimeZone).c_str());
	m_httpFacade.session().AddBodyVariable(chAttachID, Conversion::WideToUtf8(m_pSendNoteData->strAttachID).c_str());
	m_httpFacade.session().AddBodyVariable(chLang, Conversion::WideToUtf8(m_pSendNoteData->strLang).c_str());
	m_httpFacade.session().AddBodyVariable(chParty, Conversion::WideToUtf8(m_pSendNoteData->strParty).c_str());
	m_httpFacade.session().AddBodyVariable(chUserID, W2U8(pMessengerActor->GetUserID()));

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}

bool CWASSendNote::ParseAndEventFire(Json::Value * json)
{
	std::string strJsonResult = json->get("Result","").asString();
	if (strJsonResult != "OK")
	{
		m_nErrorCode = atoi(json->get("ErrorCode", "").asString().c_str());
		m_strResponseMsg = Conversion::Utf8ToWide(json->get("Message","").asString());

		return true;
	}

	return true;
}

// override in drived class
void CWASSendNote::OnWasResponse(trid_t trid,
							   bool bSuccess,
							   int /*nWinINetErrorCode*/, 
							   int /*nHttpResponseCode*/, 
                               LPCSTR szWasResCodeInHeader, 
                               LPCSTR szWasResCodeInXml,
                               Json::Value * json)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(json) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseSendNote(m_pActor, GetID(), m_eError, m_nErrorCode, m_strResponseMsg.c_str()) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->DeleteCommand(GetID());
}

void CWASSendNote::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseSendNote(m_pActor, GetID(), m_eError, m_nErrorCode, m_strResponseMsg.c_str()) == false)
			break;
	}

	//LeaveCriticalSection(pCS);
	
	if(m_pActor)
		m_pActor->DeleteCommand(GetID());
}


///////////////////////////////////////////////////////////////////////////////
//// CWASUnreadNoteCount //////////////////////////////////////////////////////

CWASUnreadNoteCount::CWASUnreadNoteCount(CWASActor *pActor) : CWASCommand(pActor)
{
    m_nUnreadNoteCount = 0;
}


CWASUnreadNoteCount::~CWASUnreadNoteCount(void)
{
}


void CWASUnreadNoteCount::ExecuteAsync()
{
	m_eError = wasError_None;
	m_nUnreadNoteCount = 0;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;

	WSTRING strURL = _T("api-nnote.ncloudbiz.com/api/note/unreadNoteCount.nhn");

	std::stringstream strStream;
	strStream << W2U8(strURL.c_str());

	strStream << "?ticket=" << pMessengerActor->GetMessengerTicket();
	strStream << "&defaultLanguage=" << "ko_KR";
	strStream << "&userid=" << W2U8(pMessengerActor->GetUserID());

	m_httpFacade.session().SetURL( strStream.str().c_str() );
	m_httpFacade.session().SetMethod(HTTP_METHOD_GET);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}


bool CWASUnreadNoteCount::ParseAndEventFire(Json::Value * json)
{
	std::string strJsonResult = json->get("Result","").asString();
	if (strJsonResult != "OK")
	{
		//m_nErrorCode = atoi(json->get("ErrorCode", "").asString().c_str());
		//m_strResponseMsg = Conversion::Utf8ToWide(json->get("Message","").asString());

		m_eError = wasError_Internal;
		return true;
	}

	m_nUnreadNoteCount = json->get("unreadNoteCount",0).asInt();

	return true;
}


// override in drived class
void CWASUnreadNoteCount::OnWasResponse(trid_t trid,
								 bool bSuccess,
								 int /*nWinINetErrorCode*/, 
								 int /*nHttpResponseCode*/, 
								 LPCSTR szWasResCodeInHeader, 
								 LPCSTR szWasResCodeInXml,
								 Json::Value * json)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(json) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseUnreadNoteCount(m_pActor, GetID(), m_eError, m_nUnreadNoteCount) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASUnreadNoteCount::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseUnreadNoteCount(m_pActor, GetID(), m_eError, m_nUnreadNoteCount) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}

//// CWASUnreadNoteCount //////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
//// CWASUnreadNoteCount //////////////////////////////////////////////////////

CWASNoteSetReadStatus::CWASNoteSetReadStatus(CWASActor *pActor) : CWASCommand(pActor)
{
	m_nNoteID = 0;
	m_strLanguage = "ko_KR";
}


CWASNoteSetReadStatus::~CWASNoteSetReadStatus(void)
{
}


void CWASNoteSetReadStatus::ExecuteAsync()
{
	m_eError = wasError_None;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;

	WSTRING strURL = _T("api-nnote.ncloudbiz.com/api/note/sendReceipt.nhn");

	std::stringstream strStream;
	strStream << W2U8(strURL.c_str());

	strStream << "?ticket=" << pMessengerActor->GetMessengerTicket();
	strStream << "&noteSedRcvSeq=" << m_nNoteID;
	strStream << "&defaultLanguage=" << m_strLanguage;
	strStream << "&userid=" << W2U8(pMessengerActor->GetUserID());

	m_httpFacade.session().SetURL( strStream.str().c_str() );
	m_httpFacade.session().SetMethod(HTTP_METHOD_GET);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}


bool CWASNoteSetReadStatus::ParseAndEventFire(Json::Value * json)
{
	std::string strJsonResult = json->get("Result","").asString();
	if (strJsonResult != "OK")
	{
		//m_nErrorCode = atoi(json->get("ErrorCode", "").asString().c_str());
		//m_strResponseMsg = Conversion::Utf8ToWide(json->get("Message","").asString());

		m_eError = wasError_Internal;
		return true;
	}

	return true;
}


// override in drived class
void CWASNoteSetReadStatus::OnWasResponse(trid_t trid,
										bool bSuccess,
										int /*nWinINetErrorCode*/, 
										int /*nHttpResponseCode*/, 
										LPCSTR szWasResCodeInHeader, 
										LPCSTR szWasResCodeInXml,
										Json::Value * json)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(json) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseNoteSetReadStatus(m_pActor, GetID(), m_eError) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASNoteSetReadStatus::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseNoteSetReadStatus(m_pActor, GetID(), m_eError) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}

//// CWASNoteSetReadStatus //////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
//// CWASGetRecvNoteInfo //////////////////////////////////////////////////////

CWASGetRecvNoteInfo::CWASGetRecvNoteInfo(CWASActor *pActor) : CWASCommand(pActor)
{
	m_nNoteID = 0;
	m_strLanguage = "ko_KR";
	m_bIncludeToList = false;
}


CWASGetRecvNoteInfo::~CWASGetRecvNoteInfo(void)
{
}


void CWASGetRecvNoteInfo::ExecuteAsync()
{
	m_eError = wasError_None;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;

	WSTRING strURL = _T("api-nnote.ncloudbiz.com/api/note/info.nhn");

	std::stringstream strStream;
	strStream << W2U8(strURL.c_str());

	strStream << "?ticket=" << pMessengerActor->GetMessengerTicket();
	strStream << "&noteSedRcvSeq=" << m_nNoteID;
	strStream << "&defaultLanguage=" << m_strLanguage;
	strStream << "&userid=" << W2U8(pMessengerActor->GetUserID());

	if( m_bIncludeToList )
		strStream << "&includeToList=true";

	m_httpFacade.session().SetURL( strStream.str().c_str() );
	m_httpFacade.session().SetMethod(HTTP_METHOD_GET);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}


bool CWASGetRecvNoteInfo::ParseAndEventFire(Json::Value * json)
{
	std::string strJsonResult = json->get("Result","").asString();
	if (strJsonResult != "OK")
	{
		//m_nErrorCode = atoi(json->get("ErrorCode", "").asString().c_str());
		//m_strResponseMsg = Conversion::Utf8ToWide(json->get("Message","").asString());

		m_eError = wasError_Internal;
		return true;
	}

	m_noteInfo.sender.userId = (*json)["noteInfo"]["from"].get("userId", "").asString();		// ߽ ̵
	m_noteInfo.sender.userName = (*json)["noteInfo"]["from"].get("userName", "").asString();	// ߽ ̸
	m_noteInfo.title = (*json)["noteInfo"].get("noteTtl", "").asString();						// 

	const Json::Value attachListJson = (*json)["noteInfo"]["attachFileInfoList"];
	for( UINT index = 0; index < attachListJson.size(); ++index )
	{
		const Json::Value attachFileInfoJson = attachListJson[index];

		AttachFileInfo attachFileInfo;
		attachFileInfo.fileName = Conversion::Utf8ToWide(attachFileInfoJson.get("rlFileNm", "").asString());
		attachFileInfo.fileSize = attachFileInfoJson.get("fileSz", "0").asUInt();				// asUInt64()  
		attachFileInfo.downloadUrl = Conversion::Utf8ToWide(attachFileInfoJson.get("downloadUrl", "").asString());

		m_noteInfo.attchFileVector.push_back(attachFileInfo);
	}

	return true;
}


// override in drived class
void CWASGetRecvNoteInfo::OnWasResponse(trid_t trid,
										  bool bSuccess,
										  int /*nWinINetErrorCode*/, 
										  int /*nHttpResponseCode*/, 
										  LPCSTR szWasResCodeInHeader, 
										  LPCSTR szWasResCodeInXml,
										  Json::Value * json)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(json) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseRecvNoteInfo(m_pActor, GetID(), m_eError, m_noteInfo, m_reqType, m_nNoteID) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	ClearNoteInfo();

	if(m_pActor)
		m_pActor->DeleteCommand(GetID());
}


void CWASGetRecvNoteInfo::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseRecvNoteInfo(m_pActor, GetID(), m_eError, m_noteInfo, m_reqType, m_nNoteID) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	ClearNoteInfo();

	if(m_pActor)
		m_pActor->DeleteCommand(GetID());
}

void CWASGetRecvNoteInfo::ClearNoteInfo()
{
	m_noteInfo.sender.userId.clear();
	m_noteInfo.sender.userName.clear();
	m_noteInfo.title.clear();
	m_noteInfo.attchFileVector.clear();
}

//// CWASGetRecvNoteInfo //////////////////////////////////////////////////////




//////////////////////////////////////////////////////////////////////////////////
//// CWASDeleteAttachedFile //////////////////////////////////////////////////////

CWASDeleteAttachedFile::CWASDeleteAttachedFile(CWASActor *pActor) : CWASCommand(pActor)
{
	m_strAttachID = "";
	m_nIndex = 0;
}


CWASDeleteAttachedFile::~CWASDeleteAttachedFile(void)
{
}


void CWASDeleteAttachedFile::ExecuteAsync()
{
	m_eError = wasError_None;
	m_bResult = false;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;
	WSTRING strURL = _T("api-nnote.ncloudbiz.com/api/note/attach/delete.nhn");
	char szRealIndex[64];
	sprintf(szRealIndex, "%d", m_nIndex);
	WSTRING strAttachId = Conversion::Utf8ToWide(m_strAttachID);
	m_httpFacade.session().SetURL( W2U8(strURL.c_str()), pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);

	m_httpFacade.session().AddBodyVariable("ticket", Conversion::URLdecode((char*)pMessengerActor->GetMessengerTicket()));
	m_httpFacade.session().AddBodyVariable("attachId",Conversion::WideToUtf8(strAttachId).c_str());
	m_httpFacade.session().AddBodyVariable("realIndex", szRealIndex);
	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}


bool CWASDeleteAttachedFile::ParseAndEventFire(Json::Value * json)
{
	std::string strJsonResult = json->get("Result","").asString();
	if (strJsonResult != "OK")
	{
		//m_nErrorCode = atoi(json->get("ErrorCode", "").asString().c_str());
		//m_strResponseMsg = Conversion::Utf8ToWide(json->get("Message","").asString());

		m_eError = wasError_Internal;
		return true;
	}
	else
		m_bResult = true;

	return true;
}


// override in drived class
void CWASDeleteAttachedFile::OnWasResponse(trid_t trid,
										bool bSuccess,
										int /*nWinINetErrorCode*/, 
										int /*nHttpResponseCode*/, 
										LPCSTR szWasResCodeInHeader, 
										LPCSTR szWasResCodeInXml,
										Json::Value * json)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(json) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseDeleteAttachedFile(m_pActor, GetID(), m_eError, m_bResult) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASDeleteAttachedFile::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseDeleteAttachedFile(m_pActor, GetID(), m_eError, m_bResult) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}

//// CWASDeleteAttachedFile //////////////////////////////////////////////////////




//////////////////////////////////////////////////////////////////////////////////
//// CWASDeleteNote //////////////////////////////////////////////////////////////

CWASDeleteNote::CWASDeleteNote(CWASActor *pActor) : CWASCommand(pActor)
{
	m_nNoteID = 0;
}


CWASDeleteNote::~CWASDeleteNote(void)
{
}


void CWASDeleteNote::ExecuteAsync()
{
	m_eError = wasError_None;
	m_bResult = false;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;

	WSTRING strURL = _T("api-nnote.ncloudbiz.com/api/note/delete.nhn");

	std::stringstream strStream;
	strStream << W2U8(strURL.c_str());

	strStream << "?ticket=" << pMessengerActor->GetMessengerTicket();
	strStream << "&noteSedRcvSeqList=" << m_nNoteID << ";";
	strStream << "&defaultLanguage=" << m_strLanguage;
	strStream << "&currentNotebxSeq=1";
	strStream << "&userid=" << W2U8(pMessengerActor->GetUserID());

	m_httpFacade.session().SetURL( strStream.str().c_str() );
	m_httpFacade.session().SetMethod(HTTP_METHOD_GET);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}


bool CWASDeleteNote::ParseAndEventFire(Json::Value * json)
{
	std::string strJsonResult = json->get("Result","").asString();
	if (strJsonResult != "OK")
	{
		//m_nErrorCode = atoi(json->get("ErrorCode", "").asString().c_str());
		//m_strResponseMsg = Conversion::Utf8ToWide(json->get("Message","").asString());

		m_eError = wasError_Internal;
		return true;
	}
	else
		m_bResult = true;

	return true;
}


// override in drived class
void CWASDeleteNote::OnWasResponse(trid_t trid,
										   bool bSuccess,
										   int /*nWinINetErrorCode*/, 
										   int /*nHttpResponseCode*/, 
										   LPCSTR szWasResCodeInHeader, 
										   LPCSTR szWasResCodeInXml,
										   Json::Value * json)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(json) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseDeleteNote(m_pActor, GetID(), m_eError, m_bResult) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASDeleteNote::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseDeleteNote(m_pActor, GetID(), m_eError, m_bResult) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}

//// CWASDeleteNote //////////////////////////////////////////////////////






////////////////////////////////////////////////////////////////////////////////////
//// CWASGetAttatchID //////////////////////////////////////////////////////////////

CWASGetAttatchID::CWASGetAttatchID(CWASActor *pActor) : CWASCommand(pActor)
{

}


CWASGetAttatchID::~CWASGetAttatchID(void)
{
}


void CWASGetAttatchID::ExecuteAsync()
{
	m_eError = wasError_None;
	m_strAttachID.clear();

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;

	WSTRING strURL = _T("api-nnote.ncloudbiz.com/api/note/attach/getId.nhn");

	std::stringstream strStream;
	strStream << W2U8(strURL.c_str());

	strStream << "?ticket=" << pMessengerActor->GetMessengerTicket();
	strStream << "&userid=" << W2U8(pMessengerActor->GetUserID());

	m_httpFacade.session().SetURL( strStream.str().c_str() );
	m_httpFacade.session().SetMethod(HTTP_METHOD_GET);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}


bool CWASGetAttatchID::ParseAndEventFire(Json::Value * json)
{
	std::string strJsonResult = json->get("Result","").asString();
	if (strJsonResult != "OK")
	{
		//m_nErrorCode = atoi(json->get("ErrorCode", "").asString().c_str());
		//m_strResponseMsg = Conversion::Utf8ToWide(json->get("Message","").asString());

		m_eError = wasError_Internal;
		return true;
	}

	m_strAttachID = (*json)["attachId"].asString();

	return true;
}


// override in drived class
void CWASGetAttatchID::OnWasResponse(trid_t trid,
								   bool bSuccess,
								   int /*nWinINetErrorCode*/, 
								   int /*nHttpResponseCode*/, 
								   LPCSTR szWasResCodeInHeader, 
								   LPCSTR szWasResCodeInXml,
								   Json::Value * json)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(json) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseGetAttachID(m_pActor, GetID(), m_eError, m_strAttachID.c_str()) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->DeleteCommand(GetID());
}


void CWASGetAttatchID::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseGetAttachID(m_pActor, GetID(), m_eError, m_strAttachID.c_str()) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->DeleteCommand(GetID());
}

//// CWASGetAttatchID //////////////////////////////////////////////////////





////////////////////////////////////////////////////////////////////////////////////
//// CWASGetReplyBody //////////////////////////////////////////////////////////////

CWASGetReplyBody::CWASGetReplyBody(CWASActor *pActor) : CWASCommand(pActor)
{
	m_nNoteID = 0;
	m_strLanguage.clear();
}


CWASGetReplyBody::~CWASGetReplyBody(void)
{
}


void CWASGetReplyBody::ExecuteAsync()
{
	m_eError = wasError_None;

	m_replyBodyInfo.strAttachID.clear();
	m_replyBodyInfo.strContent.clear();
	m_replyBodyInfo.strSubject.clear();
	m_replyBodyInfo.vecAttachFileInfo.clear();
	m_replyBodyInfo.vecReceiverVector.clear();

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;

	WSTRING strURL = _T("api-nnote.ncloudbiz.com/api/note/getWriteInfo.nhn");

	std::stringstream strStream;
	strStream << W2U8(strURL.c_str());

	strStream << "?ticket=" << pMessengerActor->GetMessengerTicket();
	strStream << "&writeMode=" << Conversion::WideToUtf8(m_strWriteMode);
	strStream << "&noteSedRcvSeq=" << m_nNoteID;
	strStream << "&defaultLanguage=" << m_strLanguage;
	strStream << "&userid=" << W2U8(pMessengerActor->GetUserID());

	m_httpFacade.session().SetURL( strStream.str().c_str() );
	m_httpFacade.session().SetMethod(HTTP_METHOD_GET);

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}


bool CWASGetReplyBody::ParseAndEventFire(Json::Value * json)
{
	std::string strJsonResult = json->get("Result","").asString();
	if (strJsonResult != "OK")
	{
		//m_nErrorCode = atoi(json->get("ErrorCode", "").asString().c_str());
		//m_strResponseMsg = Conversion::Utf8ToWide(json->get("Message","").asString());

		m_eError = wasError_Internal;
		return true;
	}

	//  
	m_replyBodyInfo.strAttachID = Conversion::Utf8ToWide((*json)["writeInfo"].get("attachId","").asString());		// utf-8 -> wide
	m_replyBodyInfo.strSubject = Conversion::Utf8ToWide((*json)["writeInfo"].get("noteTtl","").asString());			// utf-8 -> wide
	m_replyBodyInfo.strContent = Conversion::Utf8ToWide((*json)["writeInfo"].get("noteContent","").asString());		// utf-8 -> wide

	//  
	m_replyBodyInfo.vecReceiverVector.clear();
	const Json::Value child_ToList = (*json)["writeInfo"]["toList"];
	for( size_t i = 0; i < child_ToList.size(); ++i )
	{
		const Json::Value pos_Tolist = child_ToList[i];	//json ׷ ؼ;

		ReceiverInfo sReceiverInfo;
		sReceiverInfo.strID = Conversion::Utf8ToWide(pos_Tolist.get("userId","").asString()).c_str();
		sReceiverInfo.strName = Conversion::Utf8ToWide(pos_Tolist.get("userName","").asString()).c_str();
		m_replyBodyInfo.vecReceiverVector.push_back(sReceiverInfo);
	}

	// ÷ 
	
	m_replyBodyInfo.vecAttachFileInfo.clear();
	const Json::Value child2 = (*json)["writeInfo"]["attachFileInfoList"];
	for( size_t i = 0; i < child2.size(); ++i )
	{
		const Json::Value pos2 = child2[i];

		AttachFileInfo	sAttachFileInfo;
		sAttachFileInfo.fileName = Conversion::Utf8ToWide(pos2.get("rlFileNm","").asString()).c_str();
		sAttachFileInfo.fileSize = pos2.get("fileSz","0").asUInt();		// asUInt64()  
		sAttachFileInfo.downloadUrl = Conversion::Utf8ToWide(pos2.get("downloadUrl","").asString()).c_str();
		m_replyBodyInfo.vecAttachFileInfo.push_back(sAttachFileInfo);
	}

	return true;
}


// override in drived class
void CWASGetReplyBody::OnWasResponse(trid_t trid,
									 bool bSuccess,
									 int /*nWinINetErrorCode*/, 
									 int /*nHttpResponseCode*/, 
									 LPCSTR szWasResCodeInHeader, 
									 LPCSTR szWasResCodeInXml,
									 Json::Value * json)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(json) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseGetReplyBody(m_pActor, GetID(), m_eError, m_replyBodyInfo) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->DeleteCommand(GetID());
}


void CWASGetReplyBody::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseGetReplyBody(m_pActor, GetID(), m_eError, m_replyBodyInfo) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->DeleteCommand(GetID());
}

//// CWASGetReplyBody //////////////////////////////////////////////////////
